#!/usr/bin/env python
# -*- coding: utf-8 -*-

import re

# Third party Imports
from pygments import highlight
from pygments.lexers import get_lexer_by_name
from pygments.formatters import HtmlFormatter, LatexFormatter

escape_re = re.compile(r'@@(?P<contents>.+?)@@', re.MULTILINE| re.DOTALL)

html_escape_table = {
    "&": "&amp;",
    '"': "&quot;",
    "'": "&apos;",
    ">": "&gt;",
    "<": "&lt;",
    }

tex_escape_table = {
    "#": r"\#", 
    "$": r"\$", 
    "%": r"\%", 
    "&": r"\&", 
    "~": r"\~", 
    "_": r"\_", 
    "^": r"\^", 
   "\\": r"$\\backslash$", 
    "{": r"\{", 
    "}": r"\}",
    }

def html_escape(text, table = html_escape_table):
    """Produce entities within text."""
    L=[]
    for c in text:
        L.append(table.get(c,c))
    return "".join(L)

def escape_replace(match):
    contents = match.group("contents")
    return contents

class Handler(object):

    def __init__(self, rules=[]):
        self.parent = None
        self.rules = rules
        
    def callback(self, prefix, name, *args):
        method = getattr(self, prefix + name, None)
        if callable(method):
            return method(*args)
        
    def start(self, name):
        self.callback('start_', name)
    
    # not used
    def end(self, name):
        self.callback('end_', name)
        
    def sub(self, name):
        """Returns a function, ie
        sub_bold(match)
        """
        def substitution(match):
            result = self.callback('sub_', name, match)
            default = match.group(0)
            return result or default
        
        return substitution
        
class HTMLHandler(Handler):

    def checkifList(self, block):
        """
        Returns Fasle if the block is not a list
        or the html formatted block if it is.
        
        """
        islist = block.startswith("* ") or False
        envoie = False
        
        list_start = re.compile(r'(^\* )', re.MULTILINE|re.DOTALL)
        if islist :
            contents = [el.lstrip() for el in list_start.split(block) if el not in ("","\n","* ")]
            print "contents",contents
            out = ["<ul>".encode('utf-8')]
            for el in contents :
                out.append("<li>".encode('utf-8'))
                k = el.splitlines()

                for line in k:
                    line = self.tohtml(html_escape(line))
                    out.append(line.lstrip().encode('utf-8') + "<br>")
                out.append("</li>".encode('utf-8'))
            out.append("</ul>".encode('utf-8'))
            envoie = "\n".join(out)
        
        return envoie
    
    # Given a bloc, transforms it inline
    def tohtml(self,bloc):
        """
        Apply all inline rules to a given block.
        """
        for filter in self.parent.filters:
            bloc = filter(bloc, self)
        return bloc
        #return escape_re.sub(escape_replace, bloc)

        
    # Bloc substitution
    def start(self):
        # blocs = [b for b in self.parent.tokenize()]

        for name, rest in self.parent.tokenize() :
            for prefix in ('start_',) :
                method = getattr(self, prefix+name.lower(), None)
                if callable(method):
                    self.parent.sortie.append( method(rest) )

    # =======================================================================
    # HEADERS
    # =======================================================================
    def start_header(self,match):
        level = unicode(len(match[1])).encode('utf-8')
        return "<h%s>%s</h%s>"%(level, match[0].encode('utf-8'),level)
    
    # =======================================================================
    # SOURCE CODE
    # =======================================================================
    def start_code(self,match):
        lang = 'text'
        if match[1] :
            lang = match[1].lower()
        code = match[0] #.encode('utf-8')
        lexer = get_lexer_by_name(lang, stripall=True)
        formatter = HtmlFormatter(linenos=False, cssclass="perldoc")
        result = highlight(code, lexer, formatter)
        return '<pre class="code">%s</pre>'%result.encode('utf-8')

    # =======================================================================
    # PARAGRAPH
    # =======================================================================
    def start_paragraph(self,match):
        # check if it is a list
        islist = self.checkifList(match[0])
        
        if not islist:
            para = html_escape(match[0].encode('utf-8'))
            #para = "<br>".join(para.split('\n'))
            out = self.tohtml(para)
            para = "<br>".join(out.split('\n'))
            return "<p>%s</p>"%para
        else:
            return "%s"%islist
                
    # =======================================================================
    # Inline substitutions
    # =======================================================================
    def sub_bold(self, match):
        return '<strong>%s</strong>' % match.group(1).encode('utf-8')
    
    def sub_italic(self, match):
        return '<em>%s</em>' % (match.group(1)).encode('utf-8')
    
    def sub_underline(self, match):
        return '<span style="text-decoration: underline;">%s</span>' % (match.group(1)).encode('utf-8')
        
    def sub_stroked(self, match):
        return '<span style="text-decoration: line-through;">%s</span>' % (match.group(1)).encode('utf-8')
        
    def sub_exponant(self, match):
        return '<sup>%s</sup>' % (match.group(1)).encode('utf-8')
        
    def sub_indice(self, match):
        return '<sub>%s</sub>' % (match.group(1)).encode('utf-8')
        
    def sub_url(self, match):
        s = match.group(1).encode('utf-8')
        try :
            rep = match.group(2).encode('utf-8')
        except:
            rep = s
        return '<a href="%s">%s</a>' % (s,rep)
        
    def sub_mail(self, match):
        s = match.group(1).encode('utf-8')
        try :
            rep = match.group(2).encode('utf-8')
        except:
            rep = s
        return '<a href="mailto:%s">%s</a>'% (s, rep)
        
    def sub_monospace(self, match):
        return r'<tt>%s</tt>' % (match.group(1)).encode('utf-8')
        
    def sub_math(self, match):
        return r'$%s$' % (match.group(1)).encode('utf-8')
        
    def sub_hrule(self, match):
        return r'<hr/>'
        
    def sub_esc(self, match):
        return r'%s' % (match.group(1)).encode('utf-8')
        
    def sub_image(self, match):
        add = match.group(1).encode('utf-8')
        
        try :
            align = match.group(2).encode('utf-8')
        except:
            align = "center"
        
        try :
            ref = match.group(3).encode('utf-8')
        except:
            ref = add
        
        return r'<a href="%s"><div class="align-%s" align="%s"><img alt="image" class="image" src="%s"></div></a>'% (ref,align,align, add)
    
    def feed(self, data):
        return data

#=============================================================================
# LaTeX Handler
# ============================================================================

class LatexHandler(Handler):
    
    def checkifList(self, block):
        """
        Returns Fasle if the block is not a list
        or the html formatted block if it is.
        
        """
        islist = block.startswith("* ") or False
        envoie = False
        
        list_start = re.compile(r'(^\* )', re.MULTILINE|re.DOTALL)
        if islist :
            contents = [el.lstrip() for el in list_start.split(block) if el not in ("","\n","* ")]
            out = [r"\begin{itemize}".encode('utf-8')]
            for el in contents :
                out.append("\item ".encode('utf-8'))
                k = el.splitlines()

                for line in k:
                    line = self.totex(line)
                    out.append(line.lstrip().encode('utf-8'))
            out.append(r"\end{itemize}".encode('utf-8'))
            envoie = "\n".join(out)
        
        return envoie
    
    def guessHeader(self, level):
        headers = {
            1 : "chapter",
            2 : "section",
            3 : "subsection",
            4 : "subsection",
            5 : "subsection",
            6 : "subsection",
        }
        
        return headers[int(level)]

    # Given a bloc, apply all the inline transformations to it
    def totex(self,bloc):
        for filter in self.parent.filters:
            bloc = filter(bloc, self)
        return bloc

    # Bloc substitution
    def start(self):
        # blocs = [b for b in self.parent.tokenize()]

        for name, rest in self.parent.tokenize() :
            for prefix in ('start_',) :
                method = getattr(self, prefix+name.lower(), None)
                if callable(method):
                    self.parent.sortie.append( method(rest) )

    # =======================================================================
    # HEADERS
    # =======================================================================
    def start_header(self,match):
        header_title = self.guessHeader( len(match[1]) )
        return "\%s*{%s}"%(header_title.encode('utf-8'), match[0].encode('utf-8'))
    
    # =======================================================================
    # SOURCE CODE
    # =======================================================================
    def start_code(self,match):
        lang = 'text'
        if match[1] :
            lang = match[1].lower()

        code = match[0] #.! Surtout pas 'encode('utf-8')' !
        lexer = get_lexer_by_name(lang, stripall=True)
        formatter = LatexFormatter(linenos=False, cssclass="perldoc")
        result = highlight(code, lexer, formatter)
        return r'%s' % result.encode('utf-8')
        #return r'\begin{lstlisting}%s\end{lstlisting}' % result.encode('utf-8')

    # =======================================================================
    # PARAGRAPH
    # =======================================================================
    def start_paragraph(self,match):
        # check if it is a list
        islist = self.checkifList(match[0])
        
        if not islist:
            para = match[0].encode('utf-8')
            para = r"\\".join(para.split('\n'))
            out = self.totex(para)
            para = "\n".join(out.split('\n'))
            return r"%s \par"%para
        else:
            return "%s"%islist

    # =======================================================================
    # Inline substitutions
    # =======================================================================
    def sub_bold(self, match):
        return r'\textbf{%s}' % match.group(1).encode('utf-8')
    
    def sub_italic(self, match):
        return r'\textit{%s}' % (match.group(1)).encode('utf-8')
    
    def sub_underline(self, match):
        return r'\underline{%s}' % (match.group(1)).encode('utf-8')
        
    def sub_stroked(self, match):
        return r'\sout{%s}' % (match.group(1)).encode('utf-8')
        
    def sub_exponant(self, match):
        return r'$^{%s}$' % (match.group(1)).encode('utf-8')
        
    def sub_indice(self, match):
        return r'$_{%s}$' % (match.group(1)).encode('utf-8')
        
    def sub_url(self, match):
        s = match.group(1).encode('utf-8')
        return r'\url{%s}' % (s)
        
    def sub_mail(self, match):
        s = match.group(1).encode('utf-8')
        return r'\url{%s}'% (s)
        
    def sub_monospace(self, match):
        return r'\verb=%s=' % (match.group(1)).encode('utf-8')
        
    def sub_math(self, match):
        return r'$%s$' % (match.group(1)).encode('utf-8')
        
    def sub_hrule(self, match):
        return r'\hrule'
        
    def sub_image(self, match):
        return r''.encode('utf-8')
        
    def sub_esc(self, match):
        return r'%s' % (match.group(1)).encode('utf-8')
    
    def feed(self, data):
        return data
